
close all;
clear all;
clc;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Simulation of Newton's Second Law of Motion in 2D
% Used methods: Euler / Runge-Kutta 2nd/4th Order
% Author: vojta.ters@iriscores.com
% Website: https://www.iriscores.com
%
% Alternative Materials: 
% http://math.stackexchange.com/questions/721076/help-with-using-the-runge-kutta-4th-order-method-on-a-system-of-2-first-order-od
% http://lpsa.swarthmore.edu/NumInt/NumIntSecond.html
% http://introcs.cs.princeton.edu/java/94diffeq/

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Global Definitions and Constants %
global S;
S.Kappa = 6.67e-11;          % Gravitation constant [m^3 kg^-1 s^-2]
S.Mass = 5.9736e24;          % Earth Mass [kg]
S.Cl = [0,128,255]/255;      % Plotting Color
S.Cle = [102,102,102]/255;   % Earth's color
S.VelXCl = [102,0,102]/255;  % Velocity X's color
S.VelYCl = [0,102,204]/255;  % Velocity Y's colorgit commit -m ""
S.VelACl = [32,32,32]/255;   % Absolute velocity's color 
S.MSize = 12;                % Plotting Marker's size
S.Mx = 0;                    % Relative Position of the Earth [m]
S.My = 0;                    % Relative Position of the Earth [m]
S.Rz = 6378e3;               % Earth Radius [m]
S.Alt = 100e3;               % Orbitting Altitude [m]
S.Dt = 1;                    % Time step Length [s]
S.N = 100;                   % Number of points to keep for display
S.Nt = 100;                  % Invocations per one call for TimerExecFunc.

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%% Initial Conditions for Solving Differential Equation %%%%%%%
S.Vx = 9700;                 % X - Direction Speed [m/s]
S.Vy = 0;                    % Y - Direction Speed [m/s]
S.Px = 0;                    % X - Position
S.Py = S.Alt + S.Rz;         % Y - Position


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%% Reset Defaults %%%%%%%
S.MassO = S.Mass; % Earth Mass
S.MxO = S.Mx;     % Earth Pos [X]
S.MyO = S.My;     % Earth Pos [Y]
S.RzO = S.Rz ;    % Earth Diameter
S.DtO = S.Dt;     % Time Discretization [Sec]
S.VxO = S.Vx;     % Orbit Speed X-Direction [m/s]
S.VyO = S.Vy;     % Orbit Speed Y-Direction [m/s]
S.PxO = S.Px;     % Satellite Position X [m]
S.PyO = S.Py;     % Satellite Position Y [m]


S.LastPosX = S.Px*ones(1,S.N);
S.LastPosY = S.Py*ones(1,S.N);
S.LastVelX = S.Vx*ones(1,S.N);
S.LastVelY = S.Vy*ones(1,S.N);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Nt -> Its purpose is to speedup computation. The maximum calculating frequency
% of Matlab's build in timer is 1000Hz which means a maximum of 1000 computations per sec.
% This is however quite a low number, so the Nt modifies the total number of invocations per one
% Timer's function call. Nt =1 --> 1000 Invocations/sec.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%% GET DISPLAY RESOLUTION %%%%%%%
S.Displ = groot();
S.DispWidth = S.Displ.ScreenSize(3);
S.DispHeight = S.Displ.ScreenSize(4);
S.FigWidth = 1440;
S.FigHeight = 920;
S.Cls = perms([0,102,204])./255;
S.Cl_Brown      = 1;
S.Cl_Dark_Red   = 2;
S.Cl_Dark_Green = 3;
S.Cl_Violet     = 4;
S.Cl_Green      = 5;
S.Cl_Blue       = 6; 



%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%% CREATE TIMER %%%%%%%
tm = timer;
tm.TimerFcn = @Callback_Timer;
tm.tag = 'MainTimer';
tm.StartDelay = 0;
tm.ExecutionMode  = 'fixedSpacing';
tm.Period = 1/25;
tm.StartDelay = 0.5;
tm.BusyMode = 'drop';
S.tm = tm;

% COMPUTE EARTH
S.Xe = linspace(-S.Rz,S.Rz,1000);
S.Ye1 =  S.My + sqrt(S.Rz.^2 - (S.Xe).^2);
S.Ye2 =  S.My - sqrt(S.Rz.^2 - (S.Xe).^2);
S.Xe = S.Xe + S.Mx;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%% Create and Customize Figures %%%%%%%
warning('off','MATLAB:HandleGraphics:ObsoletedProperty:JavaFrame');
S.Fm = figure('name','Newton''s Second law of Motion','color',[1,1,1],'position',[S.DispWidth/2 - S.FigWidth/2,S.DispHeight/2 - S.FigHeight/2,S.FigWidth,S.FigHeight]);
javaFrame = get(S.Fm,'JavaFrame');
javaFrame.setFigureIcon(javax.swing.ImageIcon('Earth.png'));
S.Fm.NumberTitle = 'off';
S.Fm.Color = 'white';
S.Fm.Tag = 'MainFigureWindow';
S.Fm.ToolBar = 'figure';
S.Fm.MenuBar = 'none';
S.Fm.CloseRequestFcn = @Callback_Close;
S.Fm.SizeChangedFcn = @Callback_Resize;
S.Fm.ResizeFcn = @Callback_Resize;

S.Fi = figure('name','ApplicationInfo','color',[1,1,1],'Visible','off','position',[S.DispWidth/2 - S.FigWidth/2,S.DispHeight/2 - S.FigHeight/2,540,640]);
S.Fi.NumberTitle = 'off';
S.Fi.Color = 'white';
S.Fi.Tag = 'InfoFigure';
S.Fi.ToolBar = 'none';
S.Fi.MenuBar = 'none';
S.Fi.Resize = 'off';
S.Fi.CloseRequestFcn = @Callback_InfoTool_Hide;
javaFrame = get(S.Fi,'JavaFrame');
javaFrame.setFigureIcon(javax.swing.ImageIcon('Earth.png'));
Ori = S.Fi.Position;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%% Read Info From File and Assign it to Information Figure %%%%%%%
fid = fopen('Info.txt');
Inn = [];
textline = fgets(fid);
while ischar(textline)
    Inn = [Inn,textline];
    textline = fgets(fid);
end
fclose(fid);
text(0,0.5,Inn);
set(gca,'XTickLabel','');
set(gca,'YTickLabel','')
set(gca,'XTick','');
set(gca,'YTick','');
set(gca,'XColor',[1,1,1]);
set(gca,'YColor',[1,1,1]);
set(0, 'currentfigure', S.Fm);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Adjust Toolbar and get rid of Separators %
delete(findall(S.Fm,'ToolTipString','Open File'));
delete(findall(S.Fm,'ToolTipString','Print Figure'));
delete(findall(S.Fm,'ToolTipString','New Figure'));
delete(findall(S.Fm,'ToolTipString','Hide Plot Tools'));
delete(findall(S.Fm,'ToolTipString','Show Plot Tools and Dock Figure'));
delete(findall(S.Fm,'ToolTipString','Link Plot'));
delete(findall(S.Fm,'ToolTipString','Brush/Select Data'));
delete(findall(S.Fm,'ToolTipString','Edit Plot'));
delete(findall(S.Fm,'ToolTipString','Insert Legend'));
delete(findall(S.Fm,'ToolTipString','Insert ColorBar'));
Tb = findall(S.Fm,'tag','FigureToolBar');
Ty = findall(Tb,'Type','uitoggletool');
set(Ty,'Separator','off');

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%% Create Info Tool %%%%%%%
[img,map] = imread(['Images',filesep,'EarthX.gif']);
ICData = ind2rgb(img,map);
itool = uipushtool('Parent',Tb,...
    'TooltipString','About',...
    'ClickedCallBack',@Callback_InfoTool_Show,...
    'Tag','Info Box',...
    'HandleVisibility','on',...
    'CData',ICData);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%% GUI Definitions %%%%%%%%
S.AxSpacing = 20;   % Spacing Between Components
S.GS        = 0.15; % Axes Borders
S.EdWidth   = 70;   % Edit Box Width
S.EdHeight  = 20;   % Edit Box Height
S.LabWidth  = 130;  % 
S.LabHeight = 20;
S.ListBoxHeight = 30;
S.VertSpacing = 30;
S.GenOffset = 10;
S.EditHOffset = S.LabWidth + S.GenOffset;
S.ComponentsWidth = S.EdWidth + S.LabWidth + 4*S.GenOffset;
S.ComponentsHeight = 15*S.VertSpacing + 2*S.GenOffset;
S.GUIStart = S.FigHeight - S.VertSpacing - S.GenOffset;

%%%%%%%%%%%%%%%%%%%%%%%
% Create Button Group %
S.ButtonGroup1 = uibuttongroup();
S.ButtonGroup1.Position = [0 0 S.ComponentsWidth/S.FigWidth 1];
S.ButtonGroup1.Visible = 'on';
S.ButtonGroup1.Tag = 'ButtnGroup1';
S.ButtonGroup1.Title = 'Configuration';
S.ButtonGroup1.TitlePosition = 'CenterTop';

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%% CREATE EDIT BOXES %%%%%%%%
S.EditPx = uicontrol(S.ButtonGroup1,'Style','Edit','String',num2str(S.Px),...
                   'HandleVisibility','on','Tag','EditPx','HorizontalAlignment','Center');
S.EditPy = uicontrol(S.ButtonGroup1,'Style','Edit','String',num2str(S.Py),...
                   'HandleVisibility','on','Tag','EditPy','HorizontalAlignment','Center');
S.EditVx = uicontrol(S.ButtonGroup1,'Style','Edit','String',num2str(S.Vx),...
                   'HandleVisibility','on','Tag','EditVx','HorizontalAlignment','Center');
S.EditVy = uicontrol(S.ButtonGroup1,'Style','Edit','String',num2str(S.Vy),...
                   'HandleVisibility','on','Tag','EditVy','HorizontalAlignment','Center');           
S.EditDt = uicontrol(S.ButtonGroup1,'Style','Edit','String',num2str(S.Dt),...
                   'HandleVisibility','on','Tag','EditDt','HorizontalAlignment','Center');           
S.EditMass = uicontrol(S.ButtonGroup1,'Style','Edit','String',num2str(S.Mass),...
                   'HandleVisibility','on','Tag','EditMass','HorizontalAlignment','Center');
S.EditRz = uicontrol(S.ButtonGroup1,'Style','Edit','String',num2str(S.Rz),...
                   'HandleVisibility','on','Tag','EditRz','HorizontalAlignment','Center');            
S.EditMx = uicontrol(S.ButtonGroup1,'Style','Edit','String',num2str(S.Mx),...
                   'HandleVisibility','on','Tag','EditMx','HorizontalAlignment','Center');
S.EditMy = uicontrol(S.ButtonGroup1,'Style','Edit','String',num2str(S.My),...
                   'HandleVisibility','on','Tag','EditMy','HorizontalAlignment','Center');


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%% GENERATE LABELS %%%%%%%
S.LabPx = uicontrol(S.ButtonGroup1,'Style','text','String','Position Px [m]: ',...
                  'HandleVisibility','on','Tag','LabPx','HorizontalAlignment','Left',...
                  'Visible','on');
S.LabPy = uicontrol(S.ButtonGroup1,'Style','text','String','Position Py [m]: ',...
                  'HandleVisibility','on','Tag','LabPy','HorizontalAlignment','Left',...
                  'Visible','on');
S.LabVx = uicontrol(S.ButtonGroup1,'Style','text','String','Velocity Vx [m/s]: ',...
                  'HandleVisibility','on','Tag','LabVx','HorizontalAlignment','Left',...
                  'Visible','on');        
S.LabVy = uicontrol(S.ButtonGroup1,'Style','text','String','Velocity Vy [m/s]: ',...
                  'HandleVisibility','on','Tag','LabVy','HorizontalAlignment','Left',...
                  'Visible','on');
S.LabDt = uicontrol(S.ButtonGroup1,'Style','text','String','Time Step Dt [s]: ',...
                  'HandleVisibility','on','Tag','LabDt','HorizontalAlignment','Left',...
                  'Visible','on');         
S.LabMass = uicontrol(S.ButtonGroup1,'Style','text','String','Earth''s Mass [kg]: ',...
                  'HandleVisibility','on','Tag','LabMass','HorizontalAlignment','Left',...
                  'Visible','on');           
S.LabRz = uicontrol(S.ButtonGroup1,'Style','text','String','Earth''s Radius [m]: ',...
                  'HandleVisibility','on','Tag','LabRz','HorizontalAlignment','Left',...
                  'Visible','on');
S.LabMx = uicontrol(S.ButtonGroup1,'Style','text','String','Earth''s X - Position [m]: ',...
                  'HandleVisibility','on','Tag','LabMx','HorizontalAlignment','Left',...
                  'Visible','on');
S.LabMy = uicontrol(S.ButtonGroup1,'Style','text','String','Earth''s Y - Position [m]: ',...
                  'HandleVisibility','on','Tag','LabMy','HorizontalAlignment','Left',...
                  'Visible','on');

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%% Choose Differential Solver Method %%%%%%%
S.ListBox = uicontrol(S.ButtonGroup1,'Style','popupmenu',...
                   'String',{'Euler''s Method','Runge - Kutta 2nd Order','Runge - Kutta 4th Order'},...
                   'HandleVisibility','on','HorizontalAlignment','Left',...
                   'Value',3);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%% Generate Buttons %%%%%%%
S.StartBt = uicontrol(S.ButtonGroup1,'Style','pushbutton',...
               'String','Start',...
               'HandleVisibility','on',...
               'Callback',@Callback_Start_Button);
S.StopBt = uicontrol(S.ButtonGroup1,'Style','pushbutton',...
               'String','Stop',...
               'HandleVisibility','on',...
               'Callback',@Callback_Stop_Button);
S.ResetBt = uicontrol(S.ButtonGroup1,'Style','pushbutton',...
               'String','Reset',...
               'HandleVisibility','on',...
               'Callback',@Callback_Reset_Button);
           
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%% Generate Panels for Plots %%%%%%%
S.PVel = uipanel();
S.PPos = uipanel();
S.FreeWidth = S.FigWidth - S.ComponentsWidth - 3*S.AxSpacing;
S.FreeHeight = S.FigHeight - 3*S.AxSpacing;

S.PPos.Parent = S.Fm;
S.PPos.Position = [(S.ComponentsWidth + S.AxSpacing)/S.FigWidth,(2*S.AxSpacing + (S.FreeHeight/3))/S.FigHeight,(S.FreeWidth+ S.AxSpacing)/S.FigWidth,(2*S.FreeHeight/3)/S.FigHeight];
S.PPos.BackgroundColor = 'white';
S.PPos.Title = 'Position';
S.PPos.Tag = 'PositionPanel';
S.PPos.ForegroundColor = 'black';
S.PPos.FontSize = 12;

S.PVel.Parent = S.Fm;
S.PVel.Position = [(S.ComponentsWidth + S.AxSpacing )/S.FigWidth,(S.AxSpacing )/S.FigHeight,(S.FreeWidth + S.AxSpacing)/S.FigWidth,(S.FreeHeight/3)/S.FigHeight];
S.PVel.BackgroundColor = 'white';
S.PVel.Title = 'Velocities';
S.PVel.Tag = 'VelocityPanel';
S.PVel.ForegroundColor = 'black';
S.PVel.FontSize = 12;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Update Position of Components %
Callback_Resize(S.Fm,0);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%% Add Plot Areas (Axes) %%%%%%%
S.AxPos = axes();
S.AxVel = axes();

S.AxPos.Parent = S.PPos;
S.AxPos.Position = [S.GS,S.GS,1 - 2*S.GS,1-2*S.GS];
S.AxPos.Tag = 'AxPos';
S.AxPos.XGrid = 'on';
S.AxPos.YGrid = 'on';
S.AxPos.Visible = 'on';
S.AxPos.Color = 'white';
S.AxPos.Clipping = 'on';
S.AxPos.ClippingStyle = 'rectangle';

S.AxVel.Parent = S.PVel;
S.AxVel.Position = [S.GS,S.GS,1 - 2*S.GS,1-2*S.GS];
S.AxVel.Tag = 'AxVel';
S.AxVel.XGrid = 'on';
S.AxVel.YGrid = 'on';
S.AxVel.Visible = 'on';
S.AxVel.Color = 'white';
S.AxVel.Clipping = 'on';
S.AxVel.ClippingStyle = 'rectangle';

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%% Draw Sample Data %%%%%%%
Im = im2double(imread(['Images',filesep,'Earth_Map.jpg']));
axes(S.AxPos);
EarhPrecision = 1000;
[Ex,Ey,Ez] = sphere(EarhPrecision);
Exx = Ex * S.Rz + S.Mx;
Eyy = Ey * S.Rz + S.My;
Ezz = Ez * S.Rz;
S.PEarth = surf(Exx,Eyy,Ezz,'FaceColor','texturemap','EdgeColor','none','Cdata',flipud(Im));
hold on;grid on;axis equal;
S.PPos = plot(S.Px,S.Py,'.','MarkerSize',S.MSize,'color',S.Cl);
set(gca,'XTick','');
set(gca,'YTick','');
set(gca,'ZTick','');
set(gca,'XColor',[1,1,1],'YColor',[1,1,1],'TickDir','out');
view(2);
S.MiX = min(min(S.Px),-S.Rz-S.Alt + S.Mx);
S.MaX = max(max(S.Px),+S.Rz+S.Alt + S.Mx);
S.MiY = min(min(S.Py),-S.Rz-S.Alt + S.My);
S.MaY = max(max(S.Py),+S.Rz+S.Alt + S.My);
xlim([S.MiX,S.MaX]);
ylim([S.MiY,S.MaY]);
title('Newton''s Second law of Motion','color',S.Cls(S.Cl_Green,:),'FontWeight','Bold');


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%% Draw Sample Velocities Data %%%%%%%
axes(S.AxVel);
S.PVelX = plot(S.Vx,'color',S.VelXCl);
hold on; grid on;
S.PVelY = plot(S.Vy,'color',S.VelYCl);
ylabel('Velocity [m/s]');
S.PVelT = plot(sqrt(S.Vx^2 + S.Vy^2),'color',S.VelACl);
legend('X - Speed','Y - Speed','Abs Speed');
xlabel(horzcat('Time [ x ',num2str(S.Dt*S.Nt),' s] '));









