%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% iriscores.com 2016/2022
% vojta.ters@iriscores.com
% Runge - Kutta 2th order ...
% Dt ... Time Step
% X ... Position
% Vx ... Speed
% Y ... Position
% Vy ... Speed
% Kappa ... Grav. Const
% Mz ... Mass of the Earth
% Rz ... Radius of The Earth
% Mx ... Position of the Earth
% My ... Position of the Earth
%
% OUT:
% Xo    ... Computed X - Coordinate
% VXo   ... Computed X - Velocity
% Yo    ... Computed Y - Coordinate
% VYo   ... Computed Y - Velocity
% Valid ... 1 = Ok, 0 = Crashed the planet
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [ Xo, VXo, Yo, VYo,Valid ] = RK2Compute( Dt, X, Vx, Y, Vy, Kappa, Mz, Rz, Mx, My)



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Equations
% Fx = -cos(Alpha) * Kappa*Mz / (x^2 + y^2)
% Fy = -sin(Alpha) * Kappa*Mz / (x^2 + y^2)
% y(n + 1) = yn + (k1 + k2)/2; 
%
% K1 = hf( xi      , yi      ) % Beginning of Interval
% K2 = hf( xi + h  , yi + K1 ) % Endpoint of Interval
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% V(x) = d(x)/dt
% A(x) = d(v)/dt
% ------------------
% V(x) = int (A) dt
% S(x) = int (V) dt
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%



%%%%%%%%%%%%%%%%%%%%%
% Optimizations ... %
Valid = 1;
XX = X - Mx;        % Recenter Earth
YY = Y - My;        % Recenter Earth
 
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 % Avoid Crash Into Earth Surface %
if( abs(complex(XX,YY))  <= Rz)
    Valid = 0;
    VXo   = Vx;
    Xo    = X;
    VYo   = Vy;
    Yo    = Y;
else

    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %% Compute K1 Slope Slope At the Beginning of the Interval %%
    l1_x = XX; % X - Position               [S(x) = int (Vx) dt]
    l1_y = YY; % Y - Position               [S(y) = int (Vy) dt]
    v1_x = Vx; % Velocity in X - Direction  [V(x) = int (Ax) dt]
    v1_y = Vy; % Velocity in Y - Direction  [V(y) = int (Ay) dt]
    CAngle = angle(complex(l1_x,l1_y));
    Radius = (l1_x)^2 + (l1_y)^2;
    k1_x = -Kappa*Mz * cos( CAngle) / ( Radius); 
    k1_y = -Kappa*Mz * sin( CAngle) / ( Radius);
    
  
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %% Compute K2 Slope at the Endpoint of the Interval %%
    l2_x = l1_x + v1_x * Dt;    % X - Position               [S(x) = int (Vx) dt]
    l2_y = l1_y + v1_y * Dt;    % Y - Position               [S(y) = int (Vy) dt]
    v2_x = v1_x + k1_x * Dt;    % Velocity in X - Direction  [V(x) = int (Ax) dt]
    v2_y = v1_y + k1_y * Dt;    % Velocity in Y - Direction  [V(y) = int (Ay) dt]
    CAngle = angle(complex(l2_x,l2_y));
    Radius = (l2_x)^2 + (l2_y)^2;
    k2_x = -Kappa*Mz * cos(CAngle) / ( Radius);  % Acceleration in X - Direction  
    k2_y = -Kappa*Mz * sin(CAngle) / ( Radius);  % Acceleration in Y - Direction

    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %% Compute Final Slope Estimate %%
    kx = (k1_x + k2_x)/2;
    ky = (k1_y + k2_y)/2;

    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % Update Speed and Velocity based on final Estimate %
    VXo = Vx + kx * Dt;
    VYo = Vy + ky * Dt;
    Xo  = X  + Vx * Dt;
    Yo  = Y  + Vy * Dt;
end



end

