%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% iriscores.com 2016/2022
% vojta.ters@iriscores.com
% Runge - Kutta 4th order ...
% IN:
% Dt ... Time Step
% X ... Position
% Vx ... Speed
% Y ... Position
% Vy ... Speed
% Kappa ... Grav. Const
% Mz ... Mass of the Earth
% Rz ... Radius of The Earth
% Mx ... Position of the Earth
% My ... Position of the Earth
%
% OUT:
% Xo ... New X - Coordinate
% VXo ... New X - Velocity
% Yo ... New Y - Coordinate
% VYo ... New Y - Velocity
% Valid ... 1 = Ok, 0 = Crashed the planet
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [ Xo, VXo, Yo, VYo,Valid ] = RK4Compute( Dt, X, Vx, Y, Vy, Kappa, Mz, Rz, Mx, My)



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Equations
% Fx = -cos(Alpha) * Kappa*Mz / (x^2 + y^2)
% Fy = -sin(Alpha) * Kappa*Mz / (x^2 + y^2)
% y(n + 1) = yn + 1/6*(K1 + 2*K2 + 2*K3 + K4); 
%
% K1 = hf( xi       , yi        ) % Beginning of Interval
% K2 = hf( xi + h/2 , yi + k1/2 ) % Midpoint of Interval
% K3 = hf( xi + h/2 , yi + k2/2 ) % Midpoint of Interval
% K4 = hf( xi + h   , yi + k3   ) % Endpoint of Interval
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% V(x) = d(x)/dt
% A(x) = d(v)/dt
% ------------------
% V(x) = int (A) dt
% S(x) = int (V) dt
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%



%%%%%%%%%%%%%%%%%%%%%
% Optimizations ... %
Valid = 1;
XX = X - Mx;
YY = Y - My;
 
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 % Avoid Crash Into Earth Surface %
if( abs(complex(XX,YY))  <= Rz)
    Valid = 0;
    VXo   = Vx;
    Xo    = X;
    VYo   = Vy;
    Yo    = Y;
else
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %% Compute K1 Slope Slope At the Beginning of the Interval %%
    l1_x = XX; % X - Position               [S(x) = int (Vx) dt]
    l1_y = YY; % Y - Position               [S(y) = int (Vy) dt]
    v1_x = Vx; % Velocity in X - Direction  [V(x) = int (Ax) dt]
    v1_y = Vy; % Velocity in Y - Direction  [V(y) = int (Ay) dt]
    CAngle = angle(complex(l1_x,l1_y));
    Radius = (l1_x)^2 + (l1_y)^2;
    k1_x = -Kappa*Mz * cos( CAngle) / ( Radius); 
    k1_y = -Kappa*Mz * sin( CAngle) / ( Radius);

    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %% Compute K2 Slope at Midpoint while using K1 %%
    l2_x = l1_x + v1_x * Dt/2; % X - Position               [S(x) = int (Vx) dt]
    l2_y = l1_y + v1_y * Dt/2; % Y - Position               [S(y) = int (Vy) dt]
    v2_x = v1_x + k1_x * Dt/2; % Velocity in X - Direction  [V(x) = int (Ax) dt]
    v2_y = v1_y + k1_y * Dt/2; % Velocity in Y - Direction  [V(y) = int (Ay) dt]
    CAngle = angle(complex(l2_x,l2_y));
    Radius = (l2_x)^2 + (l2_y)^2;
    k2_x = -Kappa*Mz * cos( CAngle) / ( Radius);
    k2_y = -Kappa*Mz * sin( CAngle) / ( Radius);

    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %% Compute K3 Slope at Midpoint while using K2 %%
    % Note: This will basically recompute Velocity,but position will be the same.
    % The same applies for the acceleration slope therefore.
    l3_x = l1_x + v1_x * Dt/2;    % X - Position               [S(x) = int (Vx) dt]
    l3_y = l1_y + v1_y * Dt/2;    % Y - Position               [S(y) = int (Vy) dt]
    v3_x = v1_x + k2_x * Dt/2;    % Velocity in X - Direction  [V(x) = int (Ax) dt]
    v3_y = v1_y + k2_y * Dt/2;    % Velocity in Y - Direction  [V(y) = int (Ay) dt]
    k3_x = k2_x; % Same Slope Due to Same Position
    k3_y = k2_y; % Same Slope Due to Same Position

    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %% Compute K4 Slope at the endpoint while using K3 %%
    l4_x = l1_x + v1_x * Dt; % X - Position               [S(x) = int (Vx) dt]
    l4_y = l1_y + v1_y * Dt; % Y - Position               [S(y) = int (Vy) dt]
    v4_x = v1_x + v3_x * Dt; % Velocity in X - Direction  [V(x) = int (Ax) dt]
    v4_y = v1_y + v3_y * Dt; % Velocity in Y - Direction  [V(y) = int (Ay) dt]
    CAngle = angle(complex(l4_x,l4_y));
    Radius = (l4_x)^2 + (l4_y)^2;
    k4_x = -Kappa*Mz * cos( CAngle) / ( Radius); 
    k4_y = -Kappa*Mz * sin( CAngle) / ( Radius);


    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %% Compute Final Slope Estimate %%
    kx = 1/6 * (k1_x + 2*k2_x + 2*k3_x + k4_x);
    ky = 1/6 * (k1_y + 2*k2_y + 2*k3_y + k4_y);

    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % Update Speed and Velocity based on final Estimate %
    VXo = Vx + kx*Dt;
    VYo = Vy + ky*Dt;
    Xo  = X  + Vx*Dt;
    Yo  = Y  + Vy*Dt;




    % k0 = C0 * cos(angle(complex(XX,YY))) / ( (XX)^2 + (YY)^2);        % Speed  
    % l0 = Dt*Vx;                                                       % Position

    % k1 = C0 * cos(angle(complex(XX + l0,YY))) / ( (XX + l0)^2 +(YY)^2);  % Speed 
    % l1 = Dt*(Vx + k0/2);                                                     % Position

    % k2 = C0 * cos(angle(complex(XX + l1,YY))) / ( (XX + l1)^2 + (YY)^2);  % Speed 
    % l2 = Dt*(Vx + k1/2);                                                      % Position

    % k3 = C0 * cos(angle(complex(XX + l2,YY))) / ( (XX + l2)^2 + (YY)^2);  % Speed 
    % l3 = Dt*(Vx + k2);                                                    % Position

    % % SIMPSON
    % VXo = Vx + 1/6*(k0 + 2*k1 + 2*k2 + k3);
    % Xo = X   + 1/6*(l0 + 2*l1 + 2*l2 + l3);
    
    % %% Y - CALCULATION 
    % k0 = C0 * sin( angle(complex(XX,YY)))  / ((XX)^2 + (YY)^2);        % Speed 
    % l0 = Dt*Vy;                                                        % Position

    % k1 = C0 * sin( angle(complex(XX,YY + l0))) / ((XX)^2 + (YY + l0)^2);  % Speed  
    % l1 = Dt*(Vy + k0/2);                                                      % Position

    % k2 = C0 * sin( angle(complex(XX,YY + l1)))  / ((XX)^2 + (YY + l1)^2);  % Speed 
    % l2 = Dt*(Vy + k1/2);                                                       % Position

    % k3 = C0 * sin( angle(complex(XX,YY + l2))) / ( (XX)^2 + (YY + l2)^2);  % Speed 
    % l3 = Dt*(Vy + k2);                                                     % Position

    % % SIMPSON
    % VYo = Vy + 1/6*(k0 + 2*k1 + 2*k2 + k3);
    % Yo = Y   + 1/6*(l0 + 2*l1 + 2*l2 + l3);
end


end

